@@ -1,7 +1,6 @@
 import sys
-from urllib.parse import SplitResult
 import pytest
-from multidict import MultiDict, MultiDictProxy
+from urllib.parse import SplitResult
 
 from yarl import URL
 
@@ -42,6 +41,24 @@ def test_origin_no_scheme():
     with pytest.raises(ValueError):
         url.origin()
 
+
+def test_drop_dots():
+    u = URL('http://example.com/path/../to')
+    assert str(u) == 'http://example.com/to'
+
+
+def test_abs_cmp():
+    assert URL('http://example.com:8888') == URL('http://example.com:8888')
+    assert URL('http://example.com:8888/') == URL('http://example.com:8888/')
+    assert URL('http://example.com:8888/') == URL('http://example.com:8888')
+    assert URL('http://example.com:8888') == URL('http://example.com:8888/')
+
+
+def test_abs_hash():
+    url = URL('http://example.com:8888')
+    url_trailing = URL('http://example.com:8888/')
+    assert hash(url) == hash(url_trailing)
+
 # properties
 
 
@@ -150,6 +167,14 @@ def test_path_non_ascii():
     assert '/путь/сюда' == url.path
 
 
+def test_path_with_spaces():
+    url = URL('http://example.com/a b/test')
+    assert '/a b/test' == url.path
+
+    url = URL('http://example.com/a b')
+    assert '/a b' == url.path
+
+
 def test_raw_path_for_empty_url():
     url = URL()
     assert '' == url.raw_path
@@ -175,35 +200,20 @@ def test_query_string_non_ascii():
     assert url.query_string == 'б=в&ю=к'
 
 
+def test_path_qs():
+    url = URL('http://example.com/')
+    assert url.path_qs == '/'
+    url = URL('http://example.com/?б=в&ю=к')
+    assert url.path_qs == '/?б=в&ю=к'
+    url = URL('http://example.com/path?б=в&ю=к')
+    assert url.path_qs == '/path?б=в&ю=к'
+
+
 def test_query_string_spaces():
     url = URL('http://example.com?a+b=c+d&e=f+g')
     assert url.query_string == 'a b=c d&e=f g'
 
-
-def test_query_spaces():
-    url = URL('http://example.com?a+b=c+d')
-    assert url.query == MultiDict({'a b': 'c d'})
-
-
-def test_query_empty():
-    url = URL('http://example.com')
-    assert isinstance(url.query, MultiDictProxy)
-    assert url.query == MultiDict()
-
-
-def test_query():
-    url = URL('http://example.com?a=1&b=2')
-    assert url.query == MultiDict([('a', '1'), ('b', '2')])
-
-
-def test_query_repeated_args():
-    url = URL('http://example.com?a=1&b=2&a=3')
-    assert url.query == MultiDict([('a', '1'), ('b', '2'), ('a', '3')])
-
-
-def test_query_empty_arg():
-    url = URL('http://example.com?a')
-    assert url.query == MultiDict([('a', '')])
+# raw fragment
 
 
 def test_raw_fragment_empty():
@@ -334,6 +344,12 @@ def test_name_non_ascii():
     url = URL('http://example.com/путь')
     assert url.name == 'путь'
 
+
+def test_plus_in_path():
+    url = URL('http://example.com/test/x+y%2Bz/:+%2B/')
+    assert '/test/x+y+z/:++/' == url.path
+
+
 # modifiers
 
 
@@ -444,358 +460,57 @@ def test_div_with_colon_and_at():
     assert url.raw_path == '/base/path:abc@123'
 
 
-# comparison and hashing
-
-def test_ne_str():
-    url = URL('http://example.com/')
-    assert url != 'http://example.com/'
+def test_div_with_dots():
+    url = URL('http://example.com/base') / '../path/./to'
+    assert url.raw_path == '/path/to'
 
 
-def test_eq():
-    url = URL('http://example.com/')
-    assert url == URL('http://example.com/')
+# with_path
 
 
-def test_hash():
-    assert hash(URL('http://example.com/')) == hash(URL('http://example.com/'))
-
-
-def test_hash_double_call():
-    url = URL('http://example.com/')
-    assert hash(url) == hash(url)
-
-
-def test_le_less():
-    url1 = URL('http://example1.com/')
-    url2 = URL('http://example2.com/')
-
-    assert url1 <= url2
-
-
-def test_le_eq():
-    url1 = URL('http://example.com/')
-    url2 = URL('http://example.com/')
-
-    assert url1 <= url2
-
-
-def test_le_not_implemented():
-    url = URL('http://example1.com/')
-
-    assert url.__le__(123) is NotImplemented
-
-
-def test_lt():
-    url1 = URL('http://example1.com/')
-    url2 = URL('http://example2.com/')
-
-    assert url1 < url2
-
-
-def test_lt_not_implemented():
-    url = URL('http://example1.com/')
-
-    assert url.__lt__(123) is NotImplemented
-
-
-def test_ge_more():
-    url1 = URL('http://example1.com/')
-    url2 = URL('http://example2.com/')
-
-    assert url2 >= url1
-
-
-def test_ge_eq():
-    url1 = URL('http://example.com/')
-    url2 = URL('http://example.com/')
-
-    assert url2 >= url1
-
-
-def test_ge_not_implemented():
-    url = URL('http://example1.com/')
-
-    assert url.__ge__(123) is NotImplemented
-
-
-def test_gt():
-    url1 = URL('http://example1.com/')
-    url2 = URL('http://example2.com/')
-
-    assert url2 > url1
-
-
-def test_gt_not_implemented():
-    url = URL('http://example1.com/')
-
-    assert url.__gt__(123) is NotImplemented
-
-
-# with_*
-
-
-def test_with_scheme():
+def test_with_path():
     url = URL('http://example.com')
-    assert str(url.with_scheme('https')) == 'https://example.com'
+    assert str(url.with_path('/test')) == 'http://example.com/test'
 
 
-def test_with_scheme_uppercased():
+def test_with_path_encoded():
     url = URL('http://example.com')
-    assert str(url.with_scheme('HTTPS')) == 'https://example.com'
+    assert str(url.with_path('/test',
+                             encoded=True)
+               ) == 'http://example.com/test'
 
 
-def test_with_scheme_for_relative_url():
-    with pytest.raises(ValueError):
-        URL('path/to').with_scheme('http')
-
-
-def test_with_scheme_invalid_type():
+def test_with_path_dots():
     url = URL('http://example.com')
-    with pytest.raises(TypeError):
-        assert str(url.with_scheme(123))
+    assert str(url.with_path('/test/.')) == 'http://example.com/test/'
 
 
-def test_with_user():
+def test_with_path_relative():
+    url = URL('/path')
+    assert str(url.with_path('/new')) == '/new'
+
+
+def test_with_path_query():
+    url = URL('http://example.com?a=b')
+    assert str(url.with_path('/test')) == 'http://example.com/test'
+
+
+def test_with_path_fragment():
+    url = URL('http://example.com#frag')
+    assert str(url.with_path('/test')) == 'http://example.com/test'
+
+
+def test_with_path_empty():
+    url = URL('http://example.com/test')
+    assert str(url.with_path('')) == 'http://example.com'
+
+
+def test_with_path_leading_slash():
     url = URL('http://example.com')
-    assert str(url.with_user('john')) == 'http://john@example.com'
+    assert url.with_path('test').path == '/test'
 
 
-def test_with_user_non_ascii():
-    url = URL('http://example.com')
-    url2 = url.with_user('вася')
-    assert url2.raw_user == '%D0%B2%D0%B0%D1%81%D1%8F'
-    assert url2.user == 'вася'
-
-
-def test_with_user_for_relative_url():
-    with pytest.raises(ValueError):
-        URL('path/to').with_user('user')
-
-
-def test_with_user_invalid_type():
-    url = URL('http://example.com:123')
-    with pytest.raises(TypeError):
-        url.with_user(123)
-
-
-def test_with_user_None():
-    url = URL('http://john@example.com')
-    assert str(url.with_user(None)) == 'http://example.com'
-
-
-def test_with_user_None_when_password_present():
-    url = URL('http://john:pass@example.com')
-    assert str(url.with_user(None)) == 'http://example.com'
-
-
-def test_with_password():
-    url = URL('http://john@example.com')
-    assert str(url.with_password('pass')) == 'http://john:pass@example.com'
-
-
-def test_with_password_non_ascii():
-    url = URL('http://john@example.com')
-    url2 = url.with_password('пароль')
-    assert url2.raw_password == '%D0%BF%D0%B0%D1%80%D0%BE%D0%BB%D1%8C'
-    assert url2.password == 'пароль'
-
-
-def test_with_password_for_relative_url():
-    with pytest.raises(ValueError):
-        URL('path/to').with_password('pass')
-
-
-def test_with_password_None():
-    url = URL('http://john:pass@example.com')
-    assert str(url.with_password(None)) == 'http://john@example.com'
-
-
-def test_with_password_invalid_type():
-    url = URL('http://example.com:123')
-    with pytest.raises(TypeError):
-        url.with_password(123)
-
-
-def test_with_password_and_empty_user():
-    url = URL('http://example.com')
-    with pytest.raises(ValueError):
-        assert str(url.with_password('pass'))
-
-
-def test_from_str_with_host_ipv4():
-    url = URL('http://host:80')
-    url = url.with_host('192.168.1.1')
-    assert url.raw_host == '192.168.1.1'
-
-
-def test_from_str_with_host_ipv6():
-    url = URL('http://host:80')
-    url = url.with_host('::1')
-    assert url.raw_host == '::1'
-
-
-def test_with_host():
-    url = URL('http://example.com:123')
-    assert str(url.with_host('example.org')) == 'http://example.org:123'
-
-
-def test_with_host_empty():
-    url = URL('http://example.com:123')
-    with pytest.raises(ValueError):
-        url.with_host('')
-
-
-def test_with_host_non_ascii():
-    url = URL('http://example.com:123')
-    url2 = url.with_host('историк.рф')
-    assert url2.raw_host == 'xn--h1aagokeh.xn--p1ai'
-    assert url2.host == 'историк.рф'
-
-
-def test_with_host_for_relative_url():
-    with pytest.raises(ValueError):
-        URL('path/to').with_host('example.com')
-
-
-def test_with_host_invalid_type():
-    url = URL('http://example.com:123')
-    with pytest.raises(TypeError):
-        url.with_host(None)
-
-
-def test_with_port():
-    url = URL('http://example.com')
-    assert str(url.with_port(8888)) == 'http://example.com:8888'
-
-
-def test_with_port_keeps_query_and_fragment():
-    url = URL('http://example.com/?a=1#frag')
-    assert str(url.with_port(8888)) == 'http://example.com:8888/?a=1#frag'
-
-
-def test_with_port_for_relative_url():
-    with pytest.raises(ValueError):
-        URL('path/to').with_port(1234)
-
-
-def test_with_port_invalid_type():
-    with pytest.raises(TypeError):
-        URL('http://example.com').with_port('123')
-
-
-def test_with_query():
-    url = URL('http://example.com')
-    assert str(url.with_query({'a': '1'})) == 'http://example.com/?a=1'
-
-
-def test_with_query_list_of_pairs():
-    url = URL('http://example.com')
-    assert str(url.with_query([('a', '1')])) == 'http://example.com/?a=1'
-
-
-def test_with_query_kwargs():
-    url = URL('http://example.com')
-    q = url.with_query(query='1', query2='1').query
-    assert q == dict(query='1', query2='1')
-
-
-def test_with_query_kwargs_and_args_are_mutually_exclusive():
-    url = URL('http://example.com')
-    with pytest.raises(ValueError):
-        url.with_query(
-            {'a': '2', 'b': '4'}, a='1')
-
-
-def test_with_query_only_single_arg_is_supported():
-    url = URL('http://example.com')
-    u1 = url.with_query(b=3)
-    u2 = URL('http://example.com/?b=3')
-    assert u1 == u2
-    with pytest.raises(ValueError):
-        url.with_query('a=1', 'a=b')
-
-
-def test_with_query_empty_dict():
-    url = URL('http://example.com/?a=b')
-    assert str(url.with_query({})) == 'http://example.com/'
-
-
-def test_with_query_empty_str():
-    url = URL('http://example.com/?a=b')
-    assert str(url.with_query('')) == 'http://example.com/'
-
-
-def test_with_query_str():
-    url = URL('http://example.com')
-    assert str(url.with_query('a=1&b=2')) == 'http://example.com/?a=1&b=2'
-
-
-def test_with_query_str_non_ascii_and_spaces():
-    url = URL('http://example.com')
-    url2 = url.with_query('a=1 2&b=знач')
-    assert url2.raw_query_string == 'a=1+2&b=%D0%B7%D0%BD%D0%B0%D1%87'
-    assert url2.query_string == 'a=1 2&b=знач'
-
-
-def test_with_query_int():
-    url = URL('http://example.com')
-    assert url.with_query({'a': 1}) == URL('http://example.com/?a=1')
-
-
-def test_with_query_non_str():
-    url = URL('http://example.com')
-    with pytest.raises(TypeError):
-        url.with_query({'a': 1.1})
-
-
-def test_with_query_multidict():
-    url = URL('http://example.com/path')
-    q = MultiDict([('a', 'b'), ('c', 'd')])
-    assert str(url.with_query(q)) == 'http://example.com/path?a=b&c=d'
-
-
-def test_with_multidict_with_spaces_and_non_ascii():
-    url = URL('http://example.com')
-    url2 = url.with_query({'a b': 'ю б'})
-    assert url2.raw_query_string == 'a+b=%D1%8E+%D0%B1'
-
-
-def test_with_query_multidict_with_unsafe():
-    url = URL('http://example.com/path')
-    url2 = url.with_query({'a+b': '?=+&'})
-    assert url2.raw_query_string == 'a%2Bb=?%3D%2B%26'
-    assert url2.query_string == 'a%2Bb=?%3D%2B%26'
-    assert url2.query == {'a+b': '?=+&'}
-
-
-def test_with_query_None():
-    url = URL('http://example.com/path?a=b')
-    assert url.with_query(None).query_string == ''
-
-
-def test_with_query_bad_type():
-    url = URL('http://example.com')
-    with pytest.raises(TypeError):
-        url.with_query(123)
-
-
-def test_with_query_bytes():
-    url = URL('http://example.com')
-    with pytest.raises(TypeError):
-        url.with_query(b'123')
-
-
-def test_with_query_bytearray():
-    url = URL('http://example.com')
-    with pytest.raises(TypeError):
-        url.with_query(bytearray(b'123'))
-
-
-def test_with_query_memoryview():
-    url = URL('http://example.com')
-    with pytest.raises(TypeError):
-        url.with_query(memoryview(b'123'))
-
+# with_fragment
 
 def test_with_fragment():
     url = URL('http://example.com')
@@ -826,6 +541,8 @@ def test_with_fragment_bad_type():
     with pytest.raises(TypeError):
         url.with_fragment(123)
 
+# with_name
+
 
 def test_with_name():
     url = URL('http://example.com/a/b')
@@ -891,6 +608,16 @@ def test_with_name_within_colon_and_at():
     url = URL('http://example.com/oldpath').with_name('path:abc@123')
     assert url.raw_path == '/path:abc@123'
 
+
+def test_with_name_dot():
+    with pytest.raises(ValueError):
+        URL('http://example.com').with_name('.')
+
+
+def test_with_name_double_dot():
+    with pytest.raises(ValueError):
+        URL('http://example.com').with_name('..')
+
 # is_absolute
 
 
@@ -1100,7 +827,7 @@ def test_fragment_only_url():
 
 def test_url_from_url():
     url = URL('http://example.com')
-    assert URL(url) is url
+    assert URL(url) == url
     assert URL(url).raw_parts == ('/',)
 
 
